<?php

namespace Includes\baseClasses;

use Includes\model\KCPatientClinicMappingsModel;
use WP_User;
use Firebase\JWT\Key;
use Firebase\JWT\JWT;
use WP;

class KCActivate extends KCBase
{

	public static function activate()
	{
		$dependency_instance=  new KCGetDependency();
		
		//JWT Deprecated plugin
		// $dependency_instance->setPlugin('jwt-authentication-for-wp-rest-api')->getPlugin();
		$dependency_instance->setPlugin('kivicare-clinic-management-system')->getPlugin();
		$dependency_instance->setPlugin('yith-woocommerce-wishlist')->getPlugin();
		(new self())->migratePermissions();
	}

	public function init()
	{

		$this->define_table_constant();

		if (isset($_REQUEST['page']) && strpos($_REQUEST['page'], "app-configuration") !== false) {
			// Enqueue Admin-side assets...
			add_action('admin_enqueue_scripts', array($this, 'enqueueStyles'));
			add_action('admin_enqueue_scripts', array($this, 'enqueueScripts'));
		}

		// API handle
		(new KCApiHandler())->init();
		(new KCNotification());

		// Action to change authentication api response ...
		add_filter('jwt_auth_token_before_dispatch', array($this, 'jwtAuthenticationResponse'), 10, 2);

		add_action("rest_api_init", function ($wp_rest_server) {
			if (str_contains($_SERVER['REQUEST_URI'], "jwt-auth/v1/token") && isset($_SERVER['HTTP_AUTHORIZATION']) && !empty($_SERVER['HTTP_AUTHORIZATION'])) {
				unset($_SERVER['HTTP_AUTHORIZATION']);

				if (isset($_SERVER['REDIRECT_HTTP_AUTHORIZATION'])) {
					unset($_SERVER['REDIRECT_HTTP_AUTHORIZATION']);
				}
			}
		});

		// Deprecated Of JWT Plugin
		

		add_action('send_headers', function (WP $wp) {

			$headers = getallheaders();
			if (isset($headers['Authorization']) && is_plugin_active('jwt-authentication-for-wp-rest-api/jwt-auth.php')) {

				$secret_key = defined('JWT_AUTH_SECRET_KEY') ? JWT_AUTH_SECRET_KEY : false;

				$algorithm = apply_filters('jwt_auth_algorithm', 'HS256');
				$token = JWT::decode(explode(" ", $headers['Authorization'])[1], new Key($secret_key, $algorithm));

				wp_set_auth_cookie($token->data->user->id);
				wp_set_current_user($token->data->user->id);
			}
		});

		add_filter('woocommerce_rest_prepare_product_object', function ($response, $post, $request) {
			global $yith_wcwl;
			foreach ($response->data["related_ids"] as $key => $value) {
				$product = wc_get_product($value);
				$product_data = array(
					'id' => $product->get_id(),
					'name' => $product->get_name(),
					'image' => wp_get_attachment_url($product->get_image_id()),
					'price' => $product->get_price(),
					'regular_price' => $product->get_regular_price(),
					'sale_price' => $product->get_sale_price(),
					'description' => $product->get_description(),
					'average_rating' => (string) $product->get_average_rating(),
					'rating_count' => (int)$product->get_rating_count(),
				);
				$response->data["related_ids"][$key] =   $product_data;
			}
			foreach ($response->data["variations"] as $key => $value) {
				$product = wc_get_product($value);
				$product_data = array(
					'id' => $product->get_id(),
					'name' => $product->get_name(),
					'image' => wp_get_attachment_url($product->get_image_id()),
					'price' => $product->get_price(),
					'regular_price' => $product->get_regular_price(),
					'sale_price' => $product->get_sale_price(),
					'description' => $product->get_description(),
					'average_rating' => (string) $product->get_average_rating(),
					'rating_count' => (int)$product->get_rating_count(),

				);
				$response->data["variations"][$key] =   $product_data;
			}
			foreach ($response->data["grouped_products"] as $key => $value) {
				$product = wc_get_product($value);
				$product_data = array(
					'id' => $product->get_id(),
					'name' => $product->get_name(),
					'image' => wp_get_attachment_url($product->get_image_id()),
					'price' => $product->get_price(),
					'regular_price' => $product->get_regular_price(),
					'sale_price' => $product->get_sale_price(),
					'description' => $product->get_description(),
					'average_rating' => (string) $product->get_average_rating(),
					'rating_count' => (int)$product->get_rating_count(),
				);
				$response->data["grouped_products"][$key] =   $product_data;
			}

			$response->data['is_added_cart']=kc_is_product_in_cart($response->data['id']);
			$response->data['is_added_wishlist']=false;
			if (isset($yith_wcwl)) {
				$wishlist = \YITH_WCWL_Wishlist_Factory::get_wishlists(["user_id" => $this->user['id']]);
				if(isset($wishlist[0])){
					$response->data['is_added_wishlist'] = $wishlist[0]->has_product($response->data['id']);
				}
			}

			return $response;
		}, 10, 3);

		add_filter( 'woocommerce_rest_check_permissions', function(){
			if(isset($_SERVER['app_version'])){
				self::$app_version= $_SERVER['app_version'];
				return true;
			}
			return true;
		});
	}
	public function define_table_constant()
	{
		global $wpdb;
		if (!defined('KC_SERVICE_TABLE')) {
			define('KC_SERVICE_TABLE', $wpdb->prefix . 'kc_' . 'services');
		}
		if (!defined('KC_SERVICE_DOCTOR_MAPPING_TABLE')) {
			define('KC_SERVICE_DOCTOR_MAPPING_TABLE', $wpdb->prefix . 'kc_' . 'service_doctor_mapping');
		}
		if (!defined('KC_CLINIC_SCHEDULE_TABLE')) {
			define('KC_CLINIC_SCHEDULE_TABLE', $wpdb->prefix . 'kc_' . 'clinic_schedule');
		}
	}

	public function enqueueStyles()
	{
		wp_enqueue_style('kc_bootstrap_css', KIVICARE_API_DIR_URI . 'assets/css/bootstrap.min.css');
		wp_enqueue_style('kc_admin_panel_css', KIVICARE_API_DIR_URI . 'admin/css/kivicare-api-admin.css');
	}

	public function enqueueScripts()
	{
		wp_enqueue_script('kc_bootstrap_js', KIVICARE_API_DIR_URI . 'assets/js/bootstrap.min.js', ['jquery'], false, true);
	}

	public function jwtAuthenticationResponse($data, $user)
	{

		$user_info = get_userdata($user->ID);
		$basic_data  = get_user_meta($user->ID, "basic_data", true);

		$module_config = kcaGetModules();
		$date_format =  kivicareConvertMomentFormatToPhp(get_option(KIVI_CARE_PREFIX . 'date_format', "D-MM-YYYY"));

		// if( isset($user->user_status) && (int)$user->user_status === 4 ) {
		// 	$message = 'Login has been disabled. please contact you system administrator.';

		// 	return comman_message_response($message,401);
		// }
		$profile = (array) json_decode($basic_data);

		$profile['dob'] = kc_wp_date($date_format, strtotime($profile['dob']));

		if (isset($profile['specialties']) && !empty($profile['specialties'])) {
			$profile['specialties'] = array_map(function ($row) use ($date_format) {
				if (!is_null($row->created_at))
					$row->created_at = kc_wp_date($date_format, strtotime($row->created_at));
				return $row;
			}, $profile['specialties']);
		}

		/*
		if ( array_key_exists( 'profile_image', $profile ) && !empty($profile['profile_image']) ) {
			$profile['profile_image'] = wp_get_attachment_url( $profile['profile_image'] );
		}else {
			$profile['profile_image'] = null;
		}
		*/
		$data['first_name'] = $user_info->first_name;
		$data['last_name']  = $user_info->last_name;
		$data['user_id']    = $user->ID;
		$data['role']		= getUserRole($user->ID);
		$profile['profile_image'] = kcaGetProfileImage($data['role'], $user->ID);
		if ($data['role'] == 'doctor' || $data['role'] == 'receptionist') {
			$clinic_data['clinic'] = getUserClinicMapping($data);
			$data = array_merge($data, $clinic_data);
		} else {
			$patient_clinic_mappings_model = KCPatientClinicMappingsModel::find_where(['patient_id' => $user->ID]);
			$clinic_id = !is_null($patient_clinic_mappings_model) ?  $patient_clinic_mappings_model->clinic_id : kcGetDefaultClinicId();
			$clinic_data['clinic'] = [kcaGetClinic($clinic_id)];
			$data = array_merge($data, $clinic_data);
		}
		$data['isTeleMedActive'] = $this->teleMedAddOnName();

		if ($data['role'] == 'doctor') {

			if ($this->teleMedAddOnName()) {
				$config_data = kcaGetZoomConfig($user->ID);
				$data = array_merge($data, $config_data);
			}
		}

		$data['isKiviCareProOnName'] = $this->isKiviCareProOnName();

		if ($data['isKiviCareProOnName']) {
			$enableEncounter = json_decode(get_option(KIVI_CARE_PREFIX . 'enocunter_modules'));
			$enablePrescription = json_decode(get_option(KIVI_CARE_PREFIX . 'prescription_module'));

			$data['enocunter_modules'] = isset($enableEncounter->encounter_module_config) ? $enableEncounter->encounter_module_config : [];
			$data['prescription_module'] = isset($enablePrescription->prescription_module_config) ? $enablePrescription->prescription_module_config : [];

			$get_googlecal_config = get_option(KIVI_CARE_PREFIX . 'google_cal_setting', true);
			$data['is_enable_google_cal'] = $get_googlecal_config['enableCal'] == 1 ? 'on' : 'off';

			$get_patient_cal_config = get_option(KIVI_CARE_PREFIX . 'patient_cal_setting', true);
			$data['is_patient_enable'] = $get_patient_cal_config == 1 ? 'on' : 'off';
			$data['google_client_id'] = trim($get_googlecal_config['client_id']);

			if ($data['role'] == 'doctor' || $data['role'] == 'receptionist') {
				$doctor_enable = get_user_meta($data['user_id'], KIVI_CARE_PREFIX . 'google_cal_connect', true);

				$data['is_enable_doctor_gcal'] = ($doctor_enable == 'off' || empty($doctor_enable)) ? 'off' : 'on';
			}
		}
		$data = array_merge($data, (array) $module_config);
		$data = (object) array_merge($data, $profile); //(array) json_decode( $basic_data ) );
		return $data;
	}
	/**
	 * It adds a new role to the WordPress database For patient_report_edit
	 */
	public function migratePermissions()
	{
		delete_option(KIVI_CARE_PREFIX . 'permissions-migrate-1');
		if (!get_option(KIVI_CARE_PREFIX . 'permissions-migrate-1')) {
			$migrate_role_cap = array(
				array(
					"role" =>  KIVI_CARE_PREFIX . "clinic_admin",
					"cap" => array(
						KIVI_CARE_PREFIX . 'patient_report_edit' => true,
						KIVI_CARE_PREFIX . 'patient_review_add' => true,
						KIVI_CARE_PREFIX . 'patient_review_edit' => true,
						KIVI_CARE_PREFIX . 'patient_review_delete' => true,
						KIVI_CARE_PREFIX . 'patient_review_get' => true,
					)
				),
				array(
					"role" =>  KIVI_CARE_PREFIX . "doctor",
					"cap" => array(
						KIVI_CARE_PREFIX . 'patient_report_edit' => true,
						KIVI_CARE_PREFIX . 'patient_review_add' => false,
						KIVI_CARE_PREFIX . 'patient_review_edit' => false,
						KIVI_CARE_PREFIX . 'patient_review_delete' => false,
						KIVI_CARE_PREFIX . 'patient_review_get' => true,
					)
				),
				array(
					"role" => KIVI_CARE_PREFIX . "patient",
					"cap" => array(
						KIVI_CARE_PREFIX . 'patient_report_edit' => true,
						KIVI_CARE_PREFIX . 'patient_review_add' => true,
						KIVI_CARE_PREFIX . 'patient_review_edit' => true,
						KIVI_CARE_PREFIX . 'patient_review_delete' => true,
						KIVI_CARE_PREFIX . 'patient_review_get' => true,
					)
				),
				array(
					"role" =>   KIVI_CARE_PREFIX . "receptionist",
					"cap" => array(
						KIVI_CARE_PREFIX . 'patient_report_edit' => false,
						KIVI_CARE_PREFIX . 'patient_review_add' => false,
						KIVI_CARE_PREFIX . 'patient_review_edit' => false,
						KIVI_CARE_PREFIX . 'patient_review_delete' => false,
						KIVI_CARE_PREFIX . 'patient_review_get' => true,
					)
				)
			);
			foreach ($migrate_role_cap as $userRoleCap) {
				$role = get_role($userRoleCap['role']);
				foreach ($userRoleCap['cap'] as $cap => $grant) {
					$role->add_cap($cap, $grant);
				}
			}
			update_option(KIVI_CARE_PREFIX . 'permissions-migrate-1', 'yes');
		}
	}
}
