<?php

/**
 * @package  KivicarePlugin
 */

namespace Includes\baseClasses;

use WP_Error;
use WP_REST_Request;
use WP_User;

class KCBase
{

	public $plugin_path;

	public $nameSpace;

	public $plugin_url;

	public $plugin;

	public $dbConfig;

	private $pluginPrefix;

	public $plugin_version;
	public $user;
	public static $app_version;

	public function __construct()
	{

		$this->plugin_path = plugin_dir_path(dirname(__FILE__, 2));
		$this->plugin_url  = plugin_dir_url(dirname(__FILE__, 2));

		$this->nameSpace    = KIVICARE_API_NAMESPACE;
		$this->pluginPrefix = KIVICARE_API_PREFIX;

		$this->plugin_version = KIVICARE_API_VERSION;

		$this->plugin = plugin_basename(dirname(__FILE__, 3)) . '/kivicare-api.php';

		$this->dbConfig = [
			'user' => DB_USER,
			'pass' => DB_PASSWORD,
			'db'   => DB_NAME,
			'host' => DB_HOST
		];
	}

	public function kc_get_table_list()
	{
		global $wpdb;
		return [
			"KC_SERVICE_DOCTOR_MAPPING_TABLE" => $wpdb->prefix . "kc_service_doctor_mapping",
			"KC_SERVICE_TABLE" =>  $wpdb->prefix . 'kc_' . 'services',
		];
	}

	public function get_namespace()
	{
		return $this->nameSpace;
	}

	protected function getPluginPrefix()
	{
		return $this->pluginPrefix;
	}

	protected function getPrefix()
	{
		return KIVI_CARE_PREFIX;
	}

	protected function getClinicAdminRole()
	{
		return KIVI_CARE_PREFIX . "clinic_admin";
	}

	protected function getDoctorRole()
	{
		return KIVI_CARE_PREFIX . "doctor";
	}

	public function getPatientRole()
	{
		return KIVI_CARE_PREFIX . "patient";
	}
	public function getReceptionistRole()
	{
		return KIVI_CARE_PREFIX . "receptionist";
	}
	public function getLoginUserRole()
	{
		$user_id = get_current_user_id();
		$userObj = new WP_User($user_id);

		$kc_user_roles =	array_intersect(array_merge(
			['administrator'],
			$this->KCGetRoles()
		), $userObj->roles);
		$role = array_shift($kc_user_roles);
		return (!empty($role) ? $role  : '');
	}
	public function KCGetRoles()
	{
		return apply_filters('kivicare_app_user_roles', [$this->getReceptionistRole(), $this->getDoctorRole(), $this->getPatientRole()]);
	}

	protected function teleMedAddOnName()
	{

		$plugin = checkPluginExist();
		return $plugin;
	}

	public function isTeleMedActive()
	{

		if (!function_exists('get_plugins')) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugins = get_plugins();

		foreach ($plugins as $key => $value) {
			if ($value['TextDomain'] === 'kiviCare-telemed-addon') {
				return true;
			}
		}

		return false;
	}

	protected function isKiviCareProOnName()
	{

		if (!function_exists('get_plugins')) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugins = get_plugins();

		foreach ($plugins as $key => $value) {
			if ($value['TextDomain'] === 'kiviCare-clinic-&-patient-management-system-pro') {
				return (is_plugin_active($key) ? true : false);
			}
		}
		return false;
	}

	protected function isKiviCareGooglemeetActive()
	{

		if (!function_exists('get_plugins')) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugins = get_plugins();

		foreach ($plugins as $key => $value) {
			if ($value['TextDomain'] === 'kc-googlemeet') {
				return (is_plugin_active($key) ? true : false);
			}
		}
		return false;
	}

	/**
	 * If the current user doesn't have the permission to do the action, return an error
	 * 
	 * @param request The request object.
	 * 
	 * @return return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to make requests.',
	 * 'kivicare-api'), array('status' => rest_authorization_required_code()));
	 *     return true;
	 */
	public function hasPermission(WP_REST_Request $request)
	{
		/**
		 * $request->get_attributes()['callback'][1]
		 * it can get text of callback function
		 */
		$data = $this->kcaCheckAuthenticateUser($request);


		if (!$data['status']) {
			return new WP_Error('rest_forbidden', $data['massage'], array('status' => rest_authorization_required_code()));
		}

		if (!current_user_can(KIVI_CARE_PREFIX . 'patient_' . $request->get_attributes()['callback'][1])) {
			return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to make requests.', 'kivicare-api'), array('status' => rest_authorization_required_code()));
		}
		$this->user  = ['id' => get_current_user_id(), "role" => getUserRole(get_current_user_id())];

		return true;
	}
	/**
	 * If the current user doesn't have the permission to do the action, return an error
	 * 
	 * @param request The request object.
	 * 
	 * @return return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to make requests.',
	 * 'kivicare-api'), array('status' => rest_authorization_required_code()));
	 *     return true;
	 */
	public function isValidToken($request)
	{
		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return new WP_Error("Unauthorized", $data, 401);
		}
		$this->user  = ['id' => get_current_user_id(), "role" => getUserRole(get_current_user_id())];

		$this->requestParam = $request->get_params();
		return true;
	}

	public function kcaCheckAuthenticateUser($request)
	{
		$data = [
			'message' => __("User Authenticate", 'kivicare-api'),
			'status' => true,
		];

		if (!is_user_logged_in()) {
			$data['status'] = false;
			$data['message'] = __("Authorization failed", 'kivicare-api');
		} else {
			$data['user_id'] = get_current_user_id();
			$data['role'] = getUserRole($data['user_id']);
			switch_to_user_locale($data['user_id']);
		}
		return $data;
	}
}
