<?php

namespace Includes\baseClasses;

class KCNotification
{
    public $notification_data;
    public $notification_type;

    protected static $firebase_config = false;

    public function __construct()
    {
        add_action("kc_before_send_email", [$this, 'kc_send_notification'], 10, 2);
        add_action("kc_send_push_notification", [$this, 'kc_send_push_notification'], 10, 2);
    }
    public  function kc_send_notification($notification_type, $data)
    {
        if (!is_array($data)) {
            return false;
        }

        $this->notification_data = $data;
        $this->notification_type = $notification_type;

        if ($user_id = email_exists($data['user_email'])) {
            $this->kc_send_push_notification(["userID" => $user_id, "action" => $notification_type]);
        }

        if ($notification_type == "kivicare_cancel_appointment") {
            $this->kc_send_push_notification(["userID" => email_exists($data['doctor_email']), "action" => $notification_type]);
        }
    }

    public function kc_send_push_notification($obj, $skip = false)
    {

        $fields_data = $this->get_notification_data($obj);

        if (!$fields_data) return;

   
        $content = $fields_data["content"];
        $heading = $fields_data["heading"];
        $data = $fields_data["data"];
        $fields = [
            'contents'                 => $content,
            'headings'                 => $heading
        ];


        if (!function_exists('curl_init')) {
            die('cURL library is not enabled in PHP');
        }

        //function code start
        if (!self::$firebase_config) {
            $data = get_option(KIVI_CARE_PREFIX . 'onesignal_config');
            if (isset($data) && !empty($data)) {
                self::$firebase_config =$data;
            }
        }


        // FCM endpoint
        $fcmEndpoint = 'https://fcm.googleapis.com/fcm/send';

        // Notification payload
        $message = [
            'title' => $fields['headings']["en"],
            'body' => $fields['contents']["en"],
        ];

        if(getUserRole($fields_data['user_id']) === 'administrator'){
            return false;
        }

        $data = [
            'to' => "/topics/". getUserRole($fields_data['user_id'])."_".$fields_data['user_id'] ,
            'notification' => $message,
        ];

        // Set headers
        $headers = [
            'Authorization: key=' . self::$firebase_config['serverKey'],
            'Content-Type: application/json',
        ];

        // Initialize cURL session
        $ch = curl_init($fcmEndpoint);

        // Set cURL options
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));

        // Execute cURL session
        $response = curl_exec($ch);

        error_log(($response));
        // Close cURL session
        curl_close($ch);
        //function code end.

    }

    private function kc_notification_for($event = "")
    {
        return  [
            'kivicare_book_appointment' =>  [
                "heading" => esc_html__("Your Appointment Booked", "kivicare-api"),
                "content" => sprintf(__("Your appointment has been scheduled with doctor: %s", "kivicare-api"), $this->notification_data['doctor_name']),
            ],
            'kivicare_doctor_book_appointment' => [
                "heading" => esc_html__("Your Appointment Booked", "kivicare-api"),
                "content" => sprintf(__("Your appointment has been scheduled with patient: %s", "kivicare-api"),  $this->notification_data['patient_name']),
            ],
            // 'clinic_book_appointment' => [
            //     "heading" => esc_html__("Your Appointment Has Booked", "kivicare-api"),
            //     "content" => sprintf(__("Your appointment has been scheduled in your clinic with patient: %s and doctor: %s", "kivicare-api"), $event),
            // ],
            // 'zoom_link' => __(""),
            // 'meet_link' => __(""),
            // 'clinic_admin_registration' => __(""),
            // 'payment_pending' => __(""),
            // 'add_doctor_zoom_link' => __(""),
            // 'add_doctor_meet_link' => __(""),
            // 'book_appointment_reminder' => __(""),
            // 'book_prescription' => __(""),
            'kivicare_cancel_appointment' => [
                "heading" => esc_html__("Your Appointment Has Been Cancelled", "kivicare-api"),
                "content" => sprintf(__("Your appointment has been Cancelled with patient: %s and doctor: %s", "kivicare-api"), $this->notification_data['patient_name'], $this->notification_data['doctor_name']),
            ],
            'patient_clinic_check_in' => [
                "heading" => esc_html__("Patient Check In to Clinic", "kivicare-api"),
                "content" => sprintf(__("Patient %s Check In to Clinic %s", "kivicare-api"), $this->notification_data['patient_name'], $this->notification_data['clinic_name']),
            ],
            'patient_clinic_check_out' => [
                "heading" => esc_html__("Patient Check Out to Clinic", "kivicare-api"),
                "content" => sprintf(__("Patient %s Check Out to Clinic %s", "kivicare-api"), $this->notification_data['patient_name'], $this->notification_data['clinic_name']),
            ],
            // 'patient_report' => __(""),
            // // 'patient_clinic_check_in_check_out' => __(","),
            // 'admin_new_user_register' => __(""),
            // 'user_verified' => __(""),
        ];
    }
    private function get_notification_data($obj)
    {
        $player_ids = [];
        if (is_array($obj['userID'])) {
            foreach ($obj['userID'] as $id) {
                $player_ids =  array_merge($player_ids, get_user_meta($id, KIVI_CARE_PREFIX . 'firebase_tokens') ?? []);
            }
        } else {
            $player_ids = get_user_meta($obj['userID'], KIVI_CARE_PREFIX . 'firebase_tokens', true) ?? [];
        }

        $notification_action = $this->kc_notification_for()[$obj['action']] ?? false;
        
        if (is_null($player_ids) && $notification_action == false) {
            return false;
        }

        return  [
            "player_ids" => is_array($player_ids) ?  array_unique(array_values($player_ids)) : [],
            'user_id' => $obj['userID'],
            "heading" => [
                "en" => esc_html($notification_action["heading"])
            ],
            "content" => [
                "en" => esc_html($notification_action["content"])
            ],
            "data" => [
                "id" => 5645
            ]
        ];
    }
   
}
