<?php

namespace Includes\Controllers\Api;

use Includes\baseClasses\KCAppModel;
use Includes\baseClasses\KCBase;
use Includes\model\KCPatientReport;
use Includes\model\KCPatientReportModel;
use TenQuality\WP\Database\QueryBuilder;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;


class KCPatientReportController extends KCBase
{

    public $module = 'patient_report';
    public $tableName;
    public $nameSpace;

    private $requestParam;

    public function __construct()
    {
        global $wpdb;
        $this->nameSpace = KIVICARE_API_NAMESPACE;

        $this->tableName = $wpdb->prefix . "kc_patient_medical_report";

        add_action('rest_api_init', function () {
            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/add', array(
                'methods'             => WP_REST_Server::CREATABLE,
                'callback'            => [$this, 'report_add'],
                'permission_callback' => [$this, 'hasPermission'],
                'args'                => array(
                    'name'      => array(
                        'description' => __('Report Name For Of the Patients', 'kivicare-api'),
                        'required'    => true,
                        'type'        => 'string',
                        'sanitize_callback' => 'sanitize_text_field',
                    ),
                    'patient_id'   => array(
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'date' => array(
                        'default' => current_time('Y-m-d H:i:s'),
                    ),
                ),
            ));
            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/edit', array(
                'methods'             => WP_REST_Server::EDITABLE,
                'callback'            => [$this, 'report_edit'],
                'permission_callback' => [$this, 'hasPermission'],
                'args'                => array(
                    'id'      => array(
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'patient_id'   => array(
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'name'      => array(
                        'description' => __('Report Name For Of the Patients', 'kivicare-api'),
                        'required'    => true,
                        'type'        => 'string',
                        'sanitize_callback' => 'sanitize_text_field',
                        'default' => "",
                    ),

                    'date' => array(
                        'default' => current_time('Y-m-d H:i:s'),
                    ),
                ),
            ));
            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/delete', array(
                'methods'             => WP_REST_Server::DELETABLE,
                'callback'            => [$this, 'report_delete'],
                'permission_callback' => [$this, 'hasPermission'],
                'args'                => array(
                    'id'      => array(
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                ),
            ));
            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/get', array(
                'methods'             => WP_REST_Server::READABLE,
                'callback'            => [$this, 'report'],
                'permission_callback' => [$this, 'hasPermission'],
                'args'                => array(
                    'limit'   => array(
                        'default' => 8,
                        'required'    => false,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'patient_id'   => array(
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'page' => array(
                        'default' => 1,
                        'required'    => false,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'patient_id'   => array(
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),

                ),
            ));
        });
    }
    public function report_add(WP_REST_Request $request)
    {
        $files =  $request->get_file_params()['upload_report'];
        $image_handler =  media_handle_sideload($files);
        if (is_wp_error($image_handler)) {
            return comman_message_response(__("Fail To Upload Report.", 'kivicare-api'), 400);
        }


        $temp = [
            'name' => $this->requestParam['name'],
            'patient_id' => $this->requestParam['patient_id'],
            'upload_report' => $image_handler,
            'date' => $this->requestParam['date']
        ];


        $model =  KCPatientReportModel::find_where(['name' => $temp['name']]);

        if (is_null($model)) {
            if (!is_null(KCPatientReportModel::insert($temp))) {
                return comman_message_response(__("Patient Report Uploaded Successfully", 'kivicare-api'), 200);
            }
        } else {
            return comman_message_response(__("Patient Report Already Exist", 'kivicare-api'), 200);
        }
    }

    public function report_edit($request)
    {
        global $wpdb;
        $temp = [
            'name' => $this->requestParam['name'],
            'date' => $this->requestParam['date'],
        ];

        $model =  KCPatientReportModel::find_where([
            "id" => $this->requestParam['id'],
            "patient_id" => $this->requestParam['patient_id'],
        ]);

        if (is_null($model)) {
            return  comman_message_response(__("Unable To Find Patient Report", 'kivicare-api'), 200);
        }

        if ($model->update($temp)) {
            return comman_message_response(__("Your Patient Report Successfully Updated.", 'kivicare-api'), 200);
        } else {
            return comman_message_response(__("Something Went Wrong", 'kivicare-api'), 200);
        }
    }

    public function report_delete($request)
    {
        $result_model = KCPatientReportModel::delete_where([
            "id" => $this->requestParam['id'],
            "patient_id" => $this->requestParam['patient_id']
        ]);

        if ($result_model !== false)
            return comman_message_response(__("Your Patient Report Has Been Deleted.", 'kivicare-api'), 200);
        else
            return comman_message_response(__("No Patient Report Found For Delete It", 'kivicare-api'), 200);
    }


    public function report($request)
    {

        $temp = [
            'limit' => $this->requestParam['limit'],
            'offset' => ($this->requestParam['limit'] * ($this->requestParam['page'] - 1)),
            'patient_id' => $this->requestParam['patient_id'],
        ];

        $result = KCPatientReportModel::report_view($temp);

        if (empty($result)) {
            return comman_message_response(__("No Patient Report Found", 'kivicare-api'), 200);
        }
        return comman_custom_response($result, 200);
    }


    /**
     * If the current user doesn't have the permission to do the action, return an error
     * 
     * @param request The request object.
     * 
     * @return return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to make requests.',
     * 'kivicare-api'), array('status' => rest_authorization_required_code()));
     *     return true;
     */
    public function hasPermission(WP_REST_Request $request)
    {
        /**
         * $request->get_attributes()['callback'][1]
         * it can get text of callback function
         */
        $data = kcaCheckAuthenticateUser($request);

        if (!$data['status']) {
            return new WP_Error('rest_forbidden', $data['massage'], array('status' => rest_authorization_required_code()));
        }

        if (!current_user_can(KIVI_CARE_PREFIX . 'patient_' . $request->get_attributes()['callback'][1])) {
            return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to make requests.', 'kivicare-api'), array('status' => rest_authorization_required_code()));
        }
        $this->user  = ['id' => get_current_user_id(), "role" => getUserRole(get_current_user_id())];
        $this->requestParam = $request->get_params();

        return true;
    }
}
