<?php

namespace Includes\Controllers\Api;

use Includes\baseClasses\KCBase;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

class KCServiceController extends KCBase
{

	public $module = 'service';

	public $nameSpace;

	function __construct()
	{

		$this->nameSpace = KIVICARE_API_NAMESPACE;

		add_action('rest_api_init', function () {

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/get-list', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'getList'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/add-service', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'addService'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/delete-service', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'deleteService'],
				'permission_callback' => '__return_true',
			));
		});
	}

	public function addService($request)
	{
		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		$parameters = $request->get_params();

		$files =  $request->get_file_params();
		$service_imaeges = [];
		foreach ($files['doctors']['name'] ?? [] as $key => $file) {
			$service_imaeges[$key] = array(
				"name" => $files['doctors']['name'][$key]['image'],
				"type" => $files['doctors']['type'][$key]['image'],
				"tmp_name" => $files['doctors']['tmp_name'][$key]['image'],
				"error" => $files['doctors']['error'][$key]['image'],
				"size" => $files['doctors']['size'][$key]['image'],
			);
		}


		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$validation = kcaValidateRequest([
			'type' 		=> 'required',
			'name' 		=> 'required',
		], $parameters);

		if (count($validation)) {
			return comman_message_response($validation[0], 400);
		}

		$table    = $wpdb->prefix . 'kc_' . 'services';
		$mapping_table    = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';

		$temp = array(
			'type' 		=> $parameters['type'],
			'name' 		=> $parameters['name'],
			'status'	=> 1
		);

		$service_id = (isset($parameters['id']) && $parameters['id'] != '') ? $parameters['id'] : null;

		if ($service_id == null) {

			$temp['created_at'] = current_time('Y-m-d H:i:s');
			$result = $wpdb->insert($table, $temp);

			$id = $wpdb->insert_id;
			$message = __('Service has been added successfully', 'kivicare-api');
		} else {

			$condition = array('id' => $service_id);

			if (wp_query_builder()->select("*")->from("$table",false)->where($temp)->count() > 0) {
				$wpdb->update($table, $temp, $condition);
				$id = $service_id;
			} else {
				$result = $wpdb->insert($table, $temp);
				$id = $wpdb->insert_id;
			}


			$message = __('Service has been updated successfully', 'kivicare-api');
		}
		$doctor_name = [];

		if ($service_id == null) {
			foreach ($parameters['doctors'] as $key => $val) {
				$temp_data = [
					'service_id' => $id,
					'doctor_id'  => $val['doctor_id'],
					'charges'	 => $val['charges'],
					'status'	 => $val['status'] != "1" ? "0" : "1",
					"telemed_service" => $val['is_telemed'] != "1" ? "no" : "yes",
					"duration" => $val['duration'],
					"multiple" =>  $val['is_multiple_selection'] != "1" ? "no" : "yes",
				];

				if (isset($service_imaeges[$key])  && !empty($service_imaeges[$key])) {
					$temp_data['image'] =  media_handle_sideload($service_imaeges[$key]);
				}
				if ($val['charges'] == "null") {
					$doctor_name[] = get_the_author_meta('display_name', $temp_data['doctor_id']);
				} else {
					if ($data['role'] == 'receptionist') {
						$temp_data['clinic_id'] = kcGetClinicIdOfReceptionist();
					} else {
						$temp_data['clinic_id'] =  $val['clinic_id'];
					}
					$wpdb->insert($mapping_table, $temp_data);
				}
				unset($temp_data);
			}
		} else {

			// if (isset($parameters['mapping_table_id']) && !empty($parameters['mapping_table_id'])) {
			// 	foreach ($parameters['mapping_table_id'] as $key => $mapping_table_id) {
			// 		$wpdb->delete($mapping_table, ['id' => $mapping_table_id]);
			// 	}
			// }


			foreach ($parameters['doctors'] as $key => $val) {


				if (isset($parameters['service_mapping_id']) && $parameters['service_mapping_id'] != null) {
					$product_id = $this->getProductIdOfService($parameters['service_mapping_id']);
				} else {
					$product_id = $this->getProductIdOfDoctorService($parameters['id'],  $val['doctor_id']);
				}
				if ($product_id != null && get_post_status($product_id)) {
					$my_post = array(
						'ID'           => $product_id,
						'post_title'   => $parameters['name'],
					);

					wp_update_post($my_post);
				}
				if ($product_id != null && get_post_status($product_id)) {
					update_post_meta($product_id, '_price', $parameters['charges']);
					update_post_meta($product_id, '_sale_price', $parameters['charges']);
				}
				$temp_data = [
					'service_id' => $id,
					'doctor_id'  => $val['doctor_id'],
					'charges'	 => $val['charges'],
					'status'	 => $val['status'] != "1" ? "0" : "1",
					"telemed_service" => $val['is_telemed'] != "1" ? "no" : "yes",
					"duration" => $val['duration'],
					"multiple" =>  $val['is_multiple_selection'] != "1" ? "no" : "yes",
				];
				if (isset($service_imaeges[$key])  && !empty($service_imaeges[$key])) {
					$temp_data['image'] =  media_handle_sideload($service_imaeges[$key]);
				}
				$condition = array('id' => $val['mapping_table_id']);
				// if (!isset($val['mapping_table_id'])) {
				if ($data['role'] == 'receptionist') {
					$temp_data['clinic_id'] = kcGetClinicIdOfReceptionist();
				} else {
					$temp_data['clinic_id'] =  $val['clinic_id'];
				}
				if ($temp_data['charges'] == "null") {
					$doctor_name[] = get_the_author_meta('display_name', $temp_data['doctor_id']);
				} else {
				}

				$count = $wpdb->get_var("SELECT COUNT(*) FROM {$mapping_table} WHERE `doctor_id` = {$val['doctor_id']} AND  `service_id` = {$id} AND  `clinic_id` = {$temp_data['clinic_id']}  ");


				if ((int)$count > 0) {
					// return comman_message_response(__("This Service is Already Avilable in this Clinic	", "kivicare-api"));
					unset($temp_data['clinic_id']);
				}
				
				// }
				if(isset($val['mapping_table_id'])){
					$wpdb->update($mapping_table, $temp_data, $condition);
				}else{
					$wpdb->insert($mapping_table, $temp_data);
				}


				unset($temp_data);
			}
		}
		if (!empty($doctor_name)) {
			return comman_message_response(sprintf(__(" Required Field is Missing from Doctor: %s", "kivicare-api"), implode(" ", $doctor_name)), 401);
		}

		return comman_message_response($message);
	}

	public function getList(WP_REST_Request $request)
	{

		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$parameters = $request->get_params();


		$table	= $wpdb->prefix . 'kc_' . 'services';
		$user_table	= $wpdb->prefix .  'users';
		$mapping_table    = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';
		$clinic_table    = $wpdb->prefix . 'kc_' . 'clinics';

		$search	= (isset($parameters['s']) && $parameters['s'] != '') ? $parameters['s'] : null;
		$limit  = (isset($parameters['limit']) && $parameters['limit'] != '') ? $parameters['limit'] : 10;
		$page = (isset($parameters['page']) && $parameters['page'] != '') ? $parameters['page'] : 1;
		$offset = ($page - 1) *  $limit;

		$service_ids = (isset($parameters['service_ids']) && $parameters['service_ids'] != '') ? $parameters['service_ids'] : null;
		$doctor_id = (isset($parameters['doctor_id']) && $parameters['doctor_id'] != '') ? $parameters['doctor_id'] : null;
		if ($data['role'] == 'doctor') {
			$condition = " {$mapping_table}.doctor_id = {$data['user_id']} ";
			$doctor_id = $data['user_id'];
		} elseif ($doctor_id != null) {
			$condition = " {$mapping_table}.doctor_id = {$doctor_id} ";
		} else {
			$condition = " 0 = 0";
		}
		if ($doctor_id != null) {

			if ($this->teleMedAddOnName()) {
				// $zoom_config_data = get_user_meta($doctor_id, 'zoom_config_data', true);
				// $zoom_config_data = json_decode($zoom_config_data);
				// $zoom_config_data = (object) kcaGetZoomConfig($doctor_id);
				// $zoom_config_data->enableTeleMed = (bool)$zoom_config_data->enableTeleMed;
				// if (!empty($zoom_config_data->enableTeleMed) && $zoom_config_data->enableTeleMed == 'false') {
				// 	$zoom_config_data->enableTeleMed = false;
				// } else {
				// 	$zoom_config_data->enableTeleMed = true;
				// }
				// if(isset($zoom_config_data) && $zoom_config_data->enableTeleMed ) {
				// 	$condition = $condition;
				// }else{
				// 	$condition = $condition." AND {$table}.type != 'system_service' ";
				// }
				$condition = $condition;
			} else {
				$condition = $condition . " AND {$table}.type != 'system_service' ";
			}
		} else {
			$condition = $condition . " AND {$table}.type != 'system_service' ";
		}

		if (!in_array($data['role'], ['doctor', 'receptionist'])) {
			$condition .= "AND {$mapping_table}.status LIKE 1 ";
		}

		if (isset($parameters['clinic_id']) && !empty($parameters['clinic_id'])) {
			$condition .= " AND {$mapping_table}.clinic_id LIKE {$parameters['clinic_id']} ";
		}

		if ($search == null) {
			$query = "SELECT {$table}.id,{$table}.`type`,{$table}.`name`, {$mapping_table}.`status`,
						{$mapping_table}.`service_id`,{$mapping_table}.`doctor_id`,
						{$mapping_table}.`charges` , {$mapping_table}.id as `mapping_table_id`,
						{$mapping_table}.`clinic_id`,
						{$clinic_table}.`name` AS `clinic_name`, 
						{$mapping_table}.`multiple` , {$mapping_table}.`duration` , {$mapping_table}.`telemed_service`, {$mapping_table}.`image`
				FROM {$mapping_table} 
				LEFT JOIN {$table} 
					ON {$mapping_table}.`service_id` = {$table}.id 
				LEFT JOIN {$clinic_table} 
					ON {$clinic_table}.`id` = {$mapping_table}.clinic_id 
				RIGHT JOIN {$user_table} 
					ON {$mapping_table}.`doctor_id` = {$user_table}.ID 
				WHERE {$condition} ";
		} else {
			$query = "SELECT {$table}.id,{$table}.`type`,{$table}.`name`,{$table}.`status`,
						{$mapping_table}.`service_id`,{$mapping_table}.`doctor_id`,
						{$mapping_table}.`clinic_id`,
						{$clinic_table}.`name` AS `clinic_name` ,
						{$mapping_table}.`charges` , {$mapping_table}.id as `mapping_table_id`,
						{$mapping_table}.`multiple` , {$mapping_table}.`duration` , {$mapping_table}.`telemed_service` , {$mapping_table}.`image`
				FROM {$mapping_table} 
				LEFT JOIN {$table} 
					ON {$mapping_table}.`service_id` = {$table}.id
				LEFT JOIN {$clinic_table} 
					ON {$clinic_table}.`id` = {$mapping_table}.clinic_id 
				RIGHT JOIN {$user_table} 
					ON {$mapping_table}.`doctor_id` = {$user_table}.ID 
				WHERE {$condition} AND {$table}.name LIKE '{$search}%' ";
		}

		if ($service_ids != null && !empty($service_ids)) {
			$service_ids = implode(',', json_decode($service_ids));
			$query = $query . " AND {$table}.id IN ($service_ids) ";
		}
		$query = $query . " ORDER BY mapping_table_id DESC"; // LIMIT {$limit} OFFSET {$offset} ";

		if($request->has_param('with_pagination')){
			$query= $query. "LIMIT {$limit} OFFSET {$offset}";
		}


		$service_list = $wpdb->get_results($query, OBJECT);


		$service_list = array_map(function ($row) {
			$row->multiple =  $row->multiple == "yes";
			$row->telemed_service =  $row->telemed_service == "yes";
			$row->profile_image = null;

			if (!is_null($row->image)) {
				if ($img = wp_get_attachment_image_url($row->image)) {
					$row->image = $img;
				} else {
					$row->image = null;
				}
			}

			if ($url = wp_get_attachment_url(get_user_meta($row->doctor_id, 'doctor_profile_image', true))) {
				$row->profile_image = $url;
			}

			return $row;
		}, $service_list);


		if ($doctor_id == null) {
			$service_list = collect($service_list)->groupBy("name")->map(function ($item) {
				$firstItem = $item[0]; // Taking the first item in each sub-array as the main item

				return [
					'id' => $firstItem->id,
					'type' => $firstItem->type,
					'name' => $firstItem->name,
					'service_id' => $firstItem->service_id,
					'doctors' => collect($item)->unique('doctor_id')->map(function ($doctorItem) {
						return [
							'doctor_id' => $doctorItem->doctor_id,
							'display_name' => get_the_author_meta('display_name', $doctorItem->doctor_id),
							'charges' => $doctorItem->charges,
							'mapping_table_id' => $doctorItem->mapping_table_id,
							'is_multiple_selection' => $doctorItem->multiple,
							'duration' => $doctorItem->duration,
							'is_telemed' => $doctorItem->telemed_service,
							'image' => $doctorItem->image,
							'profile_image' => $doctorItem->profile_image,
							'status' => $doctorItem->status,
						];
					})->values()->toArray(), // Convert the doctor sub-collection to an array
				];
			});
		} else {
			$service_list = collect($service_list)->map(function ($row) {

				$row->is_multiple_selection = $row->multiple;
				$row->is_telemed = $row->telemed_service;

				unset($row->multiple);
				unset($row->telemed_service);

				return $row;
			})->values();
		}

		$response['total'] = $service_list->count();
		$response['data']  = $service_list->values();
		return comman_custom_response($response);
	}

	public function deleteService(WP_REST_Request $request)
	{

		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$parameters = $request->get_params();

		$table = $wpdb->prefix . 'kc_' . 'services';
		$mapping_table = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';
		if ($request->has_param('mapping_table_id')) {
			if (is_array($request->get_param('mapping_table_id'))) {
				foreach ($request->get_param('mapping_table_id') as $key => $value) {
					$results = $wpdb->delete($mapping_table, array('id' => $value));
				}
				return comman_message_response(__('Service has been deleted successfully', 'kivicare-api'));
			} else {
				$results = $wpdb->delete($mapping_table, array('id' => $request->get_param('mapping_table_id')));
				return comman_message_response(__('Service has been deleted successfully', 'kivicare-api'));
			}
		}


		if (isset($parameters['mapping_table_id']) && $parameters['mapping_table_id'] != null) {
			$product_id = $this->getProductIdOfService($parameters['mapping_table_id']);
		} else {
			$product_id = $this->getProductIdOfDoctorService($parameters['id'], $parameters['doctor_id']);
		}



		$results = $wpdb->delete($mapping_table, array('service_id' => $parameters['id'], 'doctor_id' => $parameters['doctor_id']));
		// $results = $wpdb->delete( $table , array ('id' => $parameters['id']) );
		if ($product_id != null && get_post_status($product_id)) {
			wp_delete_post($product_id);
		}
		if ($results) {
			$message = __('Service has been deleted successfully', 'kivicare-api');
		} else {
			$message = __('Service delete failed', 'kivicare-api');
		}

		return comman_message_response($message);
	}

	public function getProductIdOfService($id)
	{
		global $wpdb;
		$product_id = '';
		$appointments_service_table = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';
		$data = $wpdb->get_var('select extra from ' . $appointments_service_table . ' where id=' . $id);
		if ($data != null) {
			$data = json_decode($data);
			$product_id = $data->product_id;
		}
		return $product_id;
	}

	public function getProductIdOfDoctorService($id, $doctor_id)
	{
		global $wpdb;
		$product_id = '';
		$appointments_service_table = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';
		$data = $wpdb->get_var("SELECT extra FROM {$appointments_service_table} WHERE service_id = {$id} AND doctor_id = {$doctor_id} ");
		if ($data != null) {
			$data = json_decode($data);
			$product_id = $data->product_id;
		}
		return $product_id;
	}
}
