<?php

namespace Includes\Controllers\Api;

use Includes\baseClasses\KCBase;
use Includes\model\KCServiceDoctorMappingModel;
use Includes\model\KCTaxModel;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;


class KCTaxController    extends KCBase
{

    public $module = 'tax';
    public $tableName;
    public $nameSpace;

    public $requestParam;

    public function __construct()
    {
        global $wpdb;
        $this->nameSpace = KIVICARE_API_NAMESPACE;
        $this->tableName = $wpdb->prefix . "kc_taxes";

        add_action('rest_api_init', function () {
            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/get', array(
                'methods'             => WP_REST_Server::CREATABLE,
                'callback'            => [$this, 'get_tax'],
                'permission_callback' => [$this, 'isValidToken'],
                'args'                => array(
                    'doctor_id'   => array(
                        'description' => __('Doctor ID', 'kivicare-api'),
                        'default' => -1,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'clinic_id'   => array(
                        'description' => __('Clinic ID', 'kivicare-api'),
                        'default' => -1,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    'visit_type' => array(
                        'type'         => 'array',
                        'description' => __('Visit Type', 'kivicare-api'),
                        'required'     => true,
                        'items'        => array(
                            'type'       => 'object',
                            'properties' => array(
                                'id' => array(
                                    'type'        => 'int',
                                    'sanitize_callback' => 'absint',
                                    'description' => __('Service ID from Doctor Service Mapping', 'kivicare-api'),
                                ),
                                'quantity' => array(
                                    'type'        => 'int',
                                    'sanitize_callback' => 'absint',
                                    'description' => __('Number Of Quanity', 'kivicare-api'),
                                ),
                            ),
                        ),
                    ),
                ),
            ));
        });
    }


    /**
     * The function `get_tax` calculates the tax for a given set of services based on their charges and
     * quantities.
     * 
     * @param WP_REST_Request request The `` parameter is an instance of the `WP_REST_Request`
     * class, which represents the REST API request being made.
     * 
     * @return WP_REST_Response object containing the calculated taxes.
     */
    public function get_tax(WP_REST_Request $request)
    {
        $visit_type = collect($request->get_param('visit_type'));
        $request->set_param('service_id',$visit_type->pluck('id')->all());

        $tax = KCTaxModel::get_tax($request->get_params());
        if ($tax) {
            return new WP_REST_Response($tax);
        }
        return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to make requests.', 'kivicare-api'), array('status' => rest_authorization_required_code()));
    }
}
