<?php

namespace Includes\Controllers\Api;

use Includes\baseClasses\KCBase;
use WP_REST_Request;
use WP_REST_Server;
use YITH_WCWL_Wishlist_Factory;

class KCWishlistController extends KCBase
{

    public $module = 'kc_wishlist';

    public $nameSpace;

    function __construct()
    {

        $this->nameSpace = KIVICARE_API_NAMESPACE;

        add_action('rest_api_init', function () {

            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/add', array(
                'methods'             => WP_REST_Server::ALLMETHODS,
                'callback'            => [$this, 'kc_add_to_wishlist'],
                'permission_callback' => [$this, 'isValidToken'],
                'args' => array(
                    "product_id" => array(
                        'description' => __('Product ID for To Add Wishlist', 'kivicare-api'),
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                )
            ));

            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/get', array(
                'methods'             => WP_REST_Server::ALLMETHODS,
                'callback'            => [$this, 'kc_get_wishlist_product'],
                'permission_callback' => [$this, 'isValidToken'],
                'args' => array(
                    "per_page" => array(
                        'description' => __('Show Per Page Wishlist Products', 'kivicare-api'),
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                    "page" => array(
                        'description' => __('Show Page Wishlist Products', 'kivicare-api'),
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                )
            ));

            register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/remove', array(
                'methods'             => WP_REST_Server::ALLMETHODS,
                'callback'            => [$this, 'kc_remove_from_wishlist'],
                'permission_callback' => [$this, 'isValidToken'],
                'args' => array(
                    "product_id" => array(
                        'description' => __('Product ID for To Add Wishlist', 'kivicare-api'),
                        'required'    => true,
                        'type'        => 'int',
                        'sanitize_callback' => 'absint',
                    ),
                )
            ));
        });
    }

    public function kc_add_to_wishlist($request)
    {
        if (!function_exists("YITH_WCWL"))
            return comman_custom_response([
                "status" => false,
                "message" => __('Internal Error, Try after sometime.', 'kivicare-api')
            ]);


        $args = [
            'user_id'           => $this->user['id'],
            'add_to_wishlist'   => $request->get_param('product_id')
        ];

        try {
            YITH_WCWL()->add($args);
        } catch (\YITH_WCWL_Exception $e) {
            return comman_custom_response([
                "status" => false,
                "message" => __("Already added / Can't Process.", 'kivicare-api')
            ]);
        } catch (\Exception $e) {
            return comman_custom_response([
                "status" => false,
                "message" => __("Internal Error, Try after sometime.", 'kivicare-api')
            ]);
        }

        return comman_custom_response([
            "status" => true,
            "message" => __("Added to your wishlist", 'kivicare-api')
        ]);
    }
    public function kc_get_wishlist_product(WP_REST_Request $request)
    {
        if (!function_exists("YITH_WCWL"))
            return comman_custom_response([
                "status" => false,
                "message" => __("Internal Error, Try after sometime.", 'kivicare-api')
            ]);

        $wishlist = YITH_WCWL_Wishlist_Factory::get_wishlists(["user_id" => $this->user['id']]);
        $wishlist_id = $wishlist && isset($wishlist[0]) ? $wishlist[0]->get_data()['id'] : 0;

        $args = [
            "user_id"       => $this->user['id'],
            "wishlist_id"   => $wishlist_id
        ];

        $args["limit"] = $request->get_param('per_page');

        if (!empty($request->get_param('page'))) {
            $paged = $request->get_param('page') == 1 ? 0 : $request->get_param('page');

            if ($paged > 1) {
                $paged = ($request->get_param('per_page') * $paged - $request->get_param('per_page'));
            }
            $args["offset"] = $paged;
        }


        try {
            $results = YITH_WCWL_Wishlist_Factory::get_wishlist_items($args);
        } catch (\Exception $e) {
            return comman_custom_response([
                "status" => true,
                "message" => __("Somthing went wrong.", 'kivicare-api')
            ]);
        }

        if (empty($results))
            return comman_custom_response([
                "status" => true,
                "message" => __("Wishlist is empty", 'kivicare-api'),
                "data" => []
            ]);

        $wishlist_items = kc_get_wishlist_items($results);

        return comman_custom_response([
            "status" => true,
            "message" => __("Wishlist items", 'kivicare-api'),
            "data" => $wishlist_items
        ]);
    }
    public function kc_remove_from_wishlist(WP_REST_Request $request)
    {
        if (!function_exists("YITH_WCWL"))
            return comman_custom_response([
                "status" => false,
                "message" => __("Internal Error, Try after sometime.", 'kivicare-api')
            ]);

        $args = [
            'remove_from_wishlist'  => $request->get_param('product_id'),
            'user_id'               => $this->user['id']
        ];

        try {
            YITH_WCWL()->remove($args);
        } catch (\YITH_WCWL_Exception $e) {
            return comman_custom_response([
                "status" => false,
                "message" => __("Unable to remove, Try Again.", 'kivicare-api')
            ]);
        } catch (\Exception $e) {
            return comman_custom_response([
                "status" => false,
                "message" => __("Internal Error, Try after sometime.", 'kivicare-api')
            ]);
        }

        return comman_custom_response([
            "status" => true,
            "message" => __("Removed from your wishlist", 'kivicare-api')
        ]);
    }
}
