import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:kivicare_flutter/components/empty_error_state_component.dart';
import 'package:kivicare_flutter/components/loader_widget.dart';
import 'package:kivicare_flutter/components/no_data_found_widget.dart';
import 'package:kivicare_flutter/main.dart';
import 'package:kivicare_flutter/model/upcoming_appointment_model.dart';
import 'package:kivicare_flutter/network/appointment_repository.dart';
import 'package:kivicare_flutter/screens/appointment/appointment_functions.dart';
import 'package:kivicare_flutter/screens/appointment/components/appointment_widget.dart';
import 'package:kivicare_flutter/screens/doctor/fragments/appointment_fragment.dart';
import 'package:kivicare_flutter/components/appointment_fragment_status_compoent.dart';
import 'package:kivicare_flutter/screens/shimmer/screen/appointment_fragment_shimmer.dart';
import 'package:kivicare_flutter/utils/cached_value.dart';
import 'package:kivicare_flutter/utils/images.dart';
import 'package:nb_utils/nb_utils.dart';

class RAppointmentFragment extends StatefulWidget {
  @override
  State<RAppointmentFragment> createState() => _RAppointmentFragmentState();
}

class _RAppointmentFragmentState extends State<RAppointmentFragment> {
  Future<List<UpcomingAppointmentModel>>? future;

  StreamSubscription? updateAppointmentApi;

  List<UpcomingAppointmentModel> appointmentList = [];

  DateTime current = DateTime.now();

  int selectIndex = 0;
  int page = 1;

  bool isLastPage = false;

  @override
  void initState() {
    super.initState();
    appStore.setStatus('All');
    if (appStore.isLoading) {
      appStore.setLoading(false);
    }
    updateAppointmentApi = appointmentStreamController.stream.listen((streamData) {
      page = 1;
      init();
      setState(() {});
    });

    init();
  }

  void init({bool showLoader = false}) async {
    if (showLoader) {
      appStore.setLoading(true);
    }
    future = getReceptionistAppointmentList(
      status: appStore.mStatus,
      page: page,
      appointmentList: appointmentList,
      lastPageCallback: (p0) => isLastPage,
    ).then((value) {
      appStore.setLoading(false);
      setState(() {});
      return value;
    }).catchError((e) {
      appStore.setLoading(false);
      setState(() {});
      throw e;
    });
  }

  Future<void> changeStatus(int index) async {
    if (index == 0) {
      appStore.setStatus('All');
    } else if (index == 1) {
      appStore.setStatus('1');
    } else if (index == 2) {
      appStore.setStatus('2');
    } else if (index == 3) {
      appStore.setStatus('3');
    } else if (index == 4) {
      appStore.setStatus('0');
    } else if (index == 5) {
      appStore.setStatus('Past');
    }
    page = 1;
    init();
  }

  Future<void> _onSwipeRefresh() async {
    {
      setState(() {
        page = 1;
      });
      init();

      return await 1.seconds.delay;
    }
  }

  void _onNextPage() {
    if (!isLastPage) {
      setState(() {
        page++;
      });
      init();
    }
  }

  @override
  void setState(fn) {
    if (mounted) super.setState(fn);
  }

  @override
  void dispose() {
    if (updateAppointmentApi != null) {
      updateAppointmentApi!.cancel().then((value) {
        log("============== Stream Cancelled ==============");
      });
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Stack(
        children: [
          AppointmentFragmentStatusComponent(
            selectedIndex: selectIndex,
            callForStatusChange: (index) {
              appStore.setLoading(true);
              selectIndex = index;
              changeStatus(index);
              setState(() {});
            },
          ),
          SnapHelperWidget<List<UpcomingAppointmentModel>>(
            future: future,
            initialData: cachedReceptionistAppointment,
            loadingWidget: AppointmentFragmentShimmer(),
            errorBuilder: (error) {
              return NoDataWidget(
                imageWidget: Image.asset(
                  ic_somethingWentWrong,
                  height: 180,
                  width: 180,
                ),
                title: error.toString(),
              );
            },
            errorWidget: ErrorStateWidget(),
            onSuccess: (snap) {
              return AnimatedScrollView(
                padding: EdgeInsets.fromLTRB(16, 0, 16, 80),
                disposeScrollController: true,
                onSwipeRefresh: () => _onSwipeRefresh(),
                onNextPage: () => _onNextPage(),
                listAnimationType: listAnimationType,
                slideConfiguration: SlideConfiguration(verticalOffset: 400),
                children: snap.map((upcomingData) {
                  return AppointmentWidget(
                    upcomingData: upcomingData,
                    refreshCall: () {
                      appointmentStreamController.add(true);
                      init(showLoader: true);
                    },
                  ).paddingSymmetric(vertical: 8);
                }).toList(),
              ).visible(snap.isNotEmpty, defaultWidget: NoDataFoundWidget(text: locale.lblNoAppointmentsFound).center().visible(snap.isEmpty && !appStore.isLoading));
            },
          ).paddingTop(100),
          Observer(builder: (context) => LoaderWidget().visible(appStore.isLoading).center())
        ],
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.add),
        onPressed: () async {
          await appointmentWidgetNavigation(context).whenComplete(() {
            init(showLoader: true);
          });
        },
      ),
    );
  }
}
