import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:intl/intl.dart';
import 'package:kivicare_flutter/components/loader_widget.dart';
import 'package:kivicare_flutter/components/multi_select_service_component.dart';
import 'package:kivicare_flutter/components/role_widget.dart';
import 'package:kivicare_flutter/main.dart';
import 'package:kivicare_flutter/model/clinic_list_model.dart';
import 'package:kivicare_flutter/model/encounter_model.dart';
import 'package:kivicare_flutter/model/user_model.dart';
import 'package:kivicare_flutter/network/encounter_repository.dart';
import 'package:kivicare_flutter/screens/appointment/screen/patient_search_screen.dart';
import 'package:kivicare_flutter/screens/appointment/screen/step1_clinic_selection_screen.dart';
import 'package:kivicare_flutter/screens/appointment/screen/step2_doctor_selection_screen.dart';
import 'package:kivicare_flutter/utils/app_common.dart';
import 'package:kivicare_flutter/utils/common.dart';
import 'package:kivicare_flutter/utils/constants.dart';
import 'package:kivicare_flutter/utils/extensions/date_extensions.dart';
import 'package:kivicare_flutter/utils/extensions/string_extensions.dart';
import 'package:nb_utils/nb_utils.dart';

class AddEncounterScreen extends StatefulWidget {
  final EncounterModel? patientEncounterData;
  final int? patientId;

  AddEncounterScreen({this.patientEncounterData, this.patientId});

  @override
  _AddEncounterScreenState createState() => _AddEncounterScreenState();
}

class _AddEncounterScreenState extends State<AddEncounterScreen> {
  GlobalKey<FormState> formKey = GlobalKey();

  TextEditingController selectedClinicNameCont = TextEditingController();
  TextEditingController selectedDoctorNameCont = TextEditingController();
  TextEditingController selectedPatientNameCont = TextEditingController();
  TextEditingController encounterDateCont = TextEditingController();
  TextEditingController encounterDescriptionCont = TextEditingController();
  TextEditingController servicesCont = TextEditingController();

  EncounterModel? patientEncounterData;

  FocusNode serviceFocus = FocusNode();

  FocusNode dateFocus = FocusNode();

  Clinic? selectedClinic;

  UserModel? selectedDoctor;

  UserModel? selectedPatient;

  DateTime current = DateTime.now();

  int? clinicId;

  bool isUpdate = false;
  bool isFirstTime = true;

  @override
  void initState() {
    super.initState();
    init();
  }

  void init() async {
    isUpdate = widget.patientEncounterData != null;
    if (isUpdate) {
      patientEncounterData = widget.patientEncounterData;
      selectedPatientNameCont.text = widget.patientEncounterData!.patientName.validate();
      selectedClinicNameCont.text = widget.patientEncounterData!.clinicName.validate();
      selectedDoctorNameCont.text = widget.patientEncounterData!.doctorName.validate().prefixText(value: 'Dr. ');
      if (patientEncounterData!.encounterDate.validate().isNotEmpty) {
        current = DateFormat(SAVE_DATE_FORMAT).parse(patientEncounterData!.encounterDateGlobal.validate());
        encounterDateCont.text = current.getFormattedDate(SAVE_DATE_FORMAT);
      }

      encounterDescriptionCont.text = patientEncounterData!.description.validate();
      clinicId = patientEncounterData!.clinicId.toInt();

      setState(() {});
    }
  }

  void selectServices() async {
    await MultiSelectServiceComponent(
      clinicId: isPatient() || isDoctor() ? appointmentAppStore.mClinicSelected?.id.toInt() : userStore.userClinicId.toInt(),
      selectedServicesId: multiSelectStore.selectedService.map((element) => element.serviceId.validate()).toList(),
    ).launch(context, pageRouteAnimation: PageRouteAnimation.Fade);
    if (multiSelectStore.selectedService.length > 0)
      servicesCont.text = '${multiSelectStore.selectedService.length} ${locale.lblServicesSelected}';
    else {
      servicesCont.text = locale.lblSelectServices;
    }
  }

  @override
  void setState(fn) {
    if (mounted) super.setState(fn);
  }

  void saveEncounter() async {
    appStore.setLoading(true);

    Map request = {
      'date': encounterDateCont.text,
      "description": encounterDescriptionCont.text,
      "status": isUpdate ? widget.patientEncounterData!.status.validate() : '1',
    };

    if (isUpdate) {
      request.putIfAbsent('id', () => widget.patientEncounterData!.encounterId);
      request.putIfAbsent('patient_id', () => widget.patientEncounterData!.patientId.validate());
    } else {
      if (selectedPatient != null) request.putIfAbsent('patient_id', () => selectedPatient?.iD);
    }

    if (isDoctor()) {
      request.putIfAbsent("doctor_id", () => userStore.userId);
      if (isProEnabled()) {
        if (isUpdate) {
          request.putIfAbsent("clinic_id", () => widget.patientEncounterData!.clinicId);
        } else
          request.putIfAbsent("clinic_id", () => selectedClinic?.id);
      } else {
        request.putIfAbsent("clinic_id", () => userStore.userClinicId);
      }
    } else if (isReceptionist()) {
      request.putIfAbsent('patient_id', () => widget.patientId);
      request.putIfAbsent("clinic_id", () => userStore.userClinicId);
      request.putIfAbsent("doctor_id", () => selectedDoctor != null ? selectedDoctor?.iD : widget.patientEncounterData?.doctorId);
    }

    if (widget.patientId != null) {
      request.putIfAbsent('patient_id', () => widget.patientId);
    }

    addEncounterData(request).then((value) {
      appStore.setLoading(false);
      toast(isUpdate ? locale.lblEncounterUpdated : locale.lblAddedNewEncounter);

      finish(context, true);
    }).catchError((e) {
      appStore.setLoading(false);
      toast(e.toString());
    });
  }

  @override
  void dispose() {
    getDisposeStatusBarColor();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: appBarWidget(
        isUpdate ? locale.lblEditEncounterDetail : locale.lblAddNewEncounter,
        textColor: Colors.white,
        systemUiOverlayStyle: defaultSystemUiOverlayStyle(context),
      ),
      body: Stack(
        children: [
          Form(
            key: formKey,
            autovalidateMode: isFirstTime ? AutovalidateMode.disabled : AutovalidateMode.onUserInteraction,
            child: AnimatedScrollView(
              padding: EdgeInsets.all(16),
              children: [
                if (isProEnabled())
                  RoleWidget(
                    isShowDoctor: true,
                    child: Column(
                      children: [
                        if (widget.patientId == null)
                          AppTextField(
                            textFieldType: TextFieldType.NAME,
                            controller: selectedPatientNameCont,
                            readOnly: true,
                            decoration: inputDecoration(context: context, labelText: isUpdate ? locale.lblPatientName : locale.lblSelectPatient),
                            onTap: () async {
                              if (!isUpdate) {
                                await PatientSearchScreen().launch(context).then((value) {
                                  if (value != null) {
                                    selectedPatient = value;
                                    selectedPatientNameCont.text = selectedPatient!.userDisplayName.validate();
                                    setState(() {});
                                  }
                                });
                              } else
                                toast(locale.lblEditHolidayRestriction);
                            },
                          ),
                        16.height,
                        AppTextField(
                          controller: selectedClinicNameCont,
                          textFieldType: TextFieldType.NAME,
                          readOnly: true,
                          decoration: inputDecoration(context: context, labelText: isUpdate ? locale.lblClinicName : locale.lblSelectClinic),
                          onTap: () async {
                            if (!isUpdate) {
                              await Step1ClinicSelectionScreen(sessionOrEncounter: true, clinicId: isUpdate ? widget.patientEncounterData!.clinicId.toInt() : null).launch(context).then((value) {
                                if (value != null) {
                                  selectedClinic = value!;
                                  selectedClinicNameCont.text = selectedClinic!.name.validate();
                                } else {
                                  toast('${locale.lblPlease} ${locale.lblSelectClinic}');
                                }
                                setState(() {});
                              }).catchError((e) {
                                toast(e.toString());
                              });
                            } else
                              toast(locale.lblEditHolidayRestriction);
                          },
                        ),
                      ],
                    ),
                  ),
                if (!isDoctor()) 16.height,
                RoleWidget(
                  isShowReceptionist: true,
                  isShowPatient: true,
                  child: AppTextField(
                    textFieldType: TextFieldType.NAME,
                    controller: selectedDoctorNameCont,
                    decoration: inputDecoration(context: context, labelText: isUpdate ? locale.lblDoctorName : locale.lblSelectDoctor),
                    readOnly: true,
                    onTap: () {
                      if (!isUpdate) {
                        Step2DoctorSelectionScreen(doctorId: widget.patientEncounterData?.doctorId.toInt()).launch(context).then((value) {
                          if (value != null) {
                            selectedDoctor = value;
                            selectedDoctorNameCont.text = selectedDoctor!.displayName.validate();
                          } else {
                            toast('${locale.lblPlease} ${locale.lblSelectDoctor}');
                          }
                          setState(() {});
                        }).catchError((e) {
                          toast(e.toString());
                        });
                      }
                    },
                  ),
                ),
                16.height,
                /*AppTextField(
                  controller: servicesCont,
                  focus: serviceFocus,
                  nextFocus: dateFocus,
                  textInputAction: TextInputAction.next,
                  textFieldType: TextFieldType.NAME,
                  validator: (value) {
                    if (multiSelectStore.selectedService.isEmpty) {
                      return locale.lblSelectServices;
                    } else
                      return null;
                  },
                  decoration: inputDecoration(
                    context: context,
                    labelText: locale.lblSelectServices,
                    suffixIcon: Icon(Icons.arrow_drop_down, color: context.iconColor),
                  ).copyWith(
                    border: multiSelectStore.selectedService.isNotEmpty
                        ? OutlineInputBorder(borderSide: BorderSide.none, borderRadius: radiusOnly(topLeft: defaultRadius, topRight: defaultRadius))
                        : null,
                    enabledBorder: multiSelectStore.selectedService.isNotEmpty
                        ? OutlineInputBorder(borderSide: BorderSide.none, borderRadius: radiusOnly(topLeft: defaultRadius, topRight: defaultRadius))
                        : null,
                    focusedBorder: multiSelectStore.selectedService.isNotEmpty
                        ? OutlineInputBorder(borderSide: BorderSide.none, borderRadius: radiusOnly(topLeft: defaultRadius, topRight: defaultRadius))
                        : null,
                  ),
                  readOnly: true,
                  onTap: () {
                    if (!isUpdate) {
                      selectServices();
                    } else {
                      toast(locale.lblEditHolidayRestriction);
                    }
                  },
                ),
                if (multiSelectStore.selectedService.isNotEmpty)
                  Container(
                    padding: EdgeInsets.symmetric(vertical: 8, horizontal: 16),
                    decoration: boxDecorationDefault(color: context.cardColor, borderRadius: radiusOnly(bottomLeft: defaultRadius, bottomRight: defaultRadius)),
                    child: Column(
                      children: [
                        Row(
                          children: [
                            Text(locale.lblService.getApostropheString(count: multiSelectStore.selectedService.length, apostrophe: false), style: boldTextStyle(size: 12)).expand(),
                            Text(locale.lblCharges.getApostropheString(count: multiSelectStore.selectedService.length, apostrophe: false), style: boldTextStyle(size: 12)),
                          ],
                        ),
                        Divider(),
                        Column(
                          children: List.generate(
                            multiSelectStore.selectedService.length,
                                (index) {
                              ServiceData data = multiSelectStore.selectedService[index];
                              return Row(
                                children: [
                                  Marquee(child: Text(data.name.validate(), style: primaryTextStyle(size: 14))).expand(),
                                  PriceWidget(price: data.charges.validate().toDouble().toStringAsFixed(1), textSize: 14),
                                ],
                              ).paddingBottom(2);
                            },
                          ),
                        ),
                        Divider(),
                        Row(
                          children: [
                            Text(locale.lblTotal, style: boldTextStyle(size: 14)).expand(),
                            PriceWidget(price: multiSelectStore.selectedService.sumByDouble((p0) => p0.charges.toDouble()).toString(), textSize: 14),
                          ],
                        ),
                      ],
                    ),
                  ),*/
                AppTextField(
                  keyboardAppearance: appStore.isDarkModeOn ? Brightness.dark : Brightness.light,
                  selectionControls: EmptyTextSelectionControls(),
                  onTap: () {
                    if (isUpdate &&
                        DateFormat(SAVE_DATE_FORMAT).parse(widget.patientEncounterData!.encounterDateGlobal.validate()).isBefore(DateFormat(SAVE_DATE_FORMAT).parse(DateTime.now().toString()))) {
                      toast(locale.lblEditHolidayRestriction);
                    } else {
                      datePickerComponent(
                        context,
                        initialDate: current,
                        helpText: locale.lblSelectEncounterDate,
                        onDateSelected: (selectedDate) {
                          if (selectedDate != null) {
                            current = selectedDate;
                            encounterDateCont.text = current.getFormattedDate(SAVE_DATE_FORMAT);
                            setState(() {});
                          } else {
                            //
                          }
                        },
                      );
                    }
                  },
                  controller: encounterDateCont,
                  readOnly: true,
                  focus: dateFocus,
                  textFieldType: TextFieldType.NAME,
                  suffix: Icon(Icons.date_range),
                  decoration: inputDecoration(context: context, labelText: locale.lblDate),
                ),
                16.height,
                AppTextField(
                  controller: encounterDescriptionCont,
                  textFieldType: TextFieldType.MULTILINE,
                  isValidationRequired: false,
                  maxLines: 14,
                  textAlign: TextAlign.start,
                  minLines: 5,
                  decoration: inputDecoration(context: context, labelText: locale.lblDescription),
                ),
              ],
            ),
          ),
          Observer(builder: (context) => LoaderWidget().visible(appStore.isLoading).center())
        ],
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.done, color: Colors.white),
        onPressed: () {
          if (formKey.currentState!.validate()) {
            formKey.currentState!.save();
            showConfirmDialogCustom(
              context,
              dialogType: isUpdate ? DialogType.UPDATE : DialogType.CONFIRMATION,
              primaryColor: context.primaryColor,
              title: isUpdate ? locale.lblDoYouWantToUpdateEncounter : locale.lblDoYouWantToAddEncounter,
              onAccept: (p0) {
                saveEncounter();
              },
            );
          } else {
            isFirstTime = !isFirstTime;
            setState(() {});
          }
        },
      ),
    );
  }
}
